import 'dart:collection';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_tik_video/beans/video_tik_entry.dart';
import 'package:eyflutter_tik_video/beans/video_tik_tab.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class TikElement {
  final VideoTikEntry tikEntry;

  FijkPlayer? player;

  TikElement({required this.tikEntry, this.player});
}

class VideoTikListModel extends CusChangeNotifier {
  //播放列表
  List<TikElement> items = [];

  VideoTikTab? tikTab;

  Function(VideoTikTab tikTab, int page)? nextPage;

  Function(FijkPlayer player)? playStartNotify;

  //数据是否加载中
  bool dataLoading = false;

  ListQueue<VideoTikEntry> queue = ListQueue<VideoTikEntry>();
  bool isPreparing = false;
  Function()? initPlayerCall;
  Set<String> cacheIds = HashSet();

  TikElement element(int position) {
    if (items.length > position) {
      return items[position];
    }
    return items[0];
  }

  TikElement? currentItem() {
    if (items.isEmptyList) {
      return null;
    }
    return element(TikUtils.instance.index);
  }

  //播放视频数
  int get count => items.length;

  //视频翻页
  void videoTurn(int page) {
    if (TikUtils.instance.index == page) {
      return;
    }
    //1、播放当前视频；
    //2、保留相邻视频复原其它视频资源；
    var oldIndex = TikUtils.instance.index;
    var newIndex = page;
    recovery(oldIndex, newIndex);
    startVideo(newIndex);
    TikUtils.instance.setIndex(page);
    //触发加载下一页
    var diffCount = count - TikUtils.instance.index;
    var reserveCount = queue.length;
    var surplus = diffCount + reserveCount;
    if (surplus <= 2 && !dataLoading) {
      if (reserveCount > 0) {
        startLoadVideo();
      }
      dataLoading = true;
      nextPage?.call(tikTab ?? VideoTikTab(), newIndex);
    } else if (newIndex > oldIndex && reserveCount > 0) {
      startLoadVideo();
    }
  }

  void startVideo(int newIndex) {
    var entry = element(newIndex);
    if (entry.player?.state == FijkState.idle) {
      resetPlayer(entry.tikEntry, entry.player);
    }
    TikUtils.instance.start(entry.player);
  }

  void recovery(int oldIndex, int newIndex) {
    var index = -1;
    if (oldIndex > newIndex) {
      index = oldIndex + 2;
    } else if (oldIndex < newIndex) {
      index = oldIndex - 1;
    }
    if (index >= 0 && (index + 1) < count) {
      var tikElement = element(oldIndex);
      tikElement.player?.reset();
    }
    var item = element(oldIndex);
    TikUtils.instance.recoveryPause(item.player);
  }

  /// 完成数据装载
  void complete() {
    dataLoading = false;
  }

  //逐个添加视频待准备完成
  appendVideos(List<VideoTikEntry> list) {
    for (var entry in list) {
      if (cacheIds.contains(entry.id)) {
        continue;
      }
      __prepareVideo(entry);
      // queue.addFirst(entry);
    }
    // startLoadVideo();
    complete();
  }

  void startLoadVideo() {
    if (!isPreparing) {
      _prepareVideo();
    }
  }

  __prepareVideo(VideoTikEntry entry) {
    var tikElement = TikElement(
      tikEntry: entry,
      player: instancePlayer(entry),
    );
    items.add(tikElement);
    cacheIds.add(tikElement.tikEntry.id);
    notifierLazy();
  }

  _prepareVideo() {
    if (queue.isEmpty) {
      isPreparing = false;
      complete();
      return;
    }
    isPreparing = true;
    VideoTikEntry removeLast = queue.removeLast();
    if (cacheIds.contains(removeLast.id)) {
      _prepareVideo();
      return;
    }
    var tikElement = TikElement(
      tikEntry: removeLast,
      player: instancePlayer(removeLast),
    );
    initPlayerListener(() {
      if (tikElement.player?.state == FijkState.prepared) {
        tikElement.player?.removeListener(initPlayerCall!);
        items.add(tikElement);
        cacheIds.add(tikElement.tikEntry.id);
        notifierLazy();
        isPreparing = false;
        _prepareVideo();
      } else if (tikElement.player?.state == FijkState.error) {
        //视频资源无效加载下一个视频
        isPreparing = false;
        _prepareVideo();
      }
    });
    tikElement.player?.addListener(initPlayerCall!);
  }

  void initPlayerListener(VoidCallback initCall) {
    initPlayerCall = initCall;
  }

  void resetPlayer(VideoTikEntry entry, FijkPlayer? player) {
    player?.setDataSource(entry.videoUrl,
        autoPlay: entry.autoPlay, showCover: true);
  }

  FijkPlayer instancePlayer(VideoTikEntry entry) {
    var player = FijkPlayer();
    Function()? playerCall;
    playerCall  = () {
      if (player.state == FijkState.started) {
        playStartNotify!(player);
      }
    };
    player.addListener(playerCall);
    Future.wait([
      //通过立即清理数据包来减少等待时长
      player.setOption(FijkOption.formatCategory, "flush_packets", 1),
      player.setOption(FijkOption.formatCategory, "probesize", 1024 * 16),
      //设置最长分析时长
      player.setOption(FijkOption.formatCategory, "analyzemaxduration", 100),
      // player.setOption(FijkOption.formatCategory, "http-detect-range-support", 0),
      //跳过循环滤波
      player.setOption(FijkOption.codecCategory, "skip_loop_filter", 48),
      player.setOption(FijkOption.codecCategory, "skip_frame", 0),

      // player.setOption(FijkOption.playerCategory, "max_cached_duration", 500);
      // player.setOption(FijkOption.playerCategory, "dns_cache_clear", 0);
      player.setOption(FijkOption.playerCategory, "reconnect", 3),
      // player.setOption(FijkOption.playerCategory, "mediacodec", 1);
      // player.setOption(FijkOption.playerCategory, "mediacodec-auto-rotate", 1);
      // player.setOption(FijkOption.playerCategory, "mediacodec-handle-resolution-change", 1);
      //网络不好的情况下进行丢包
      player.setOption(FijkOption.playerCategory, "framedrop", 1),
      player.setOption(FijkOption.playerCategory, "infbuf", 1),
      //暂停输出直到停止后读取足够的数据包
      player.setOption(FijkOption.playerCategory, "packet-buffering", 0),
      // 不查询stream_info，直接使用
      player.setOption(FijkOption.playerCategory, "find_stream_info", 1),
      //等待开始之后才绘制
      player.setOption(FijkOption.playerCategory, "render-wait-start", 0),
    ]).then((result){
        player.setDataSource(
          entry.videoUrl,
          showCover: true,
          autoPlay: entry.autoPlay,
        );
        // player.addListener(() {
          // if (player.state == FijkState.error) {
          //   resetPlayer(entry,player);
          // }
        // });
    });
    return player;
  }

  //根据页面切换改变当前视频播放状态
  void changeCurrentPlayState(int tabIndex) {
    var index = TikUtils.instance.getIndex(tabIndex);
    if (items.length >= (index + 1)) {
      var tikElement = element(index);
      if (TikUtils.instance.getFrontState(tabIndex)) {
        TikUtils.instance.start(tikElement.player);
      } else {
        TikUtils.instance.pause(tikElement.player);
      }
    }
  }

  //回收视频资源
  void release() {
    for (var item in items) {
      item.player?.release();
    }
  }
}
