import 'dart:async';

import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_tik_video/beans/video_tik_entry.dart';
import 'package:eyflutter_tik_video/models/collect_model.dart';
import 'package:eyflutter_tik_video/models/portrait_model.dart';
import 'package:eyflutter_tik_video/models/share_model.dart';
import 'package:eyflutter_tik_video/models/video_loading_model.dart';
import 'package:eyflutter_tik_video/models/video_play_progress_model.dart';
import 'package:eyflutter_tik_video/models/video_player_icon_model.dart';
import 'package:eyflutter_tik_video/models/video_tik_list_model.dart';
import 'package:eyflutter_tik_video/tiklist/favorite_animation_icon.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';
import 'package:percent_indicator/percent_indicator.dart';

mixin TikVideoListener {
  //点赞
  void onFavoriteCall();

  //分享
  void onShareCall(ShareModel model);

  //收藏
  void onCollectCall(CollectModel model);

  //头像
  void onPortraitCall(PortraitModel model);
}

/// 单击：暂停；双击：点赞，双击后紧接着一次单击也是点赞；
class TikVideoGesture extends StatefulWidget {
  /// 构建完成回调
  final Function() buildCall;

  /// 进度条颜色
  final Color progressColor;

  /// 进度条背景
  final Color progressBackgroundColor;

  /// 填充方式
  final FijkFit fit;

  final TikElement element;

  final TikVideoListener? listener;

  final int position;

  final int initSeek;

  final int tabIndex;

  const TikVideoGesture({
    Key? key,
    required this.buildCall,
    required this.progressColor,
    required this.progressBackgroundColor,
    required this.fit,
    required this.element,
    this.listener,
    required this.position,
    required this.initSeek,
    required this.tabIndex,
  }) : super(key: key);

  @override
  _TikVideoGestureState createState() => _TikVideoGestureState();
}

class _TikVideoGestureState extends State<TikVideoGesture> {
  List<Offset> icons = [];
  bool canAddFavorite = false;
  bool justAddFavorite = false;
  GlobalKey key = GlobalKey();
  Timer? timer;
  PortraitModel portraitModel = PortraitModel();
  VideoPlayProgressModel progressModel = VideoPlayProgressModel();
  VideoPlayerIconModel playerIconModel = VideoPlayerIconModel();
  VideoLoadingModel loadingModel = VideoLoadingModel();
  CollectModel collectModel = CollectModel();
  ShareModel shareModel = ShareModel();

  @override
  void initState() {
    super.initState();
    widget.buildCall.call();
    CloudEBus.instance.addEvent(this, action: "playIconSetting", method: onPlayIconSetting);
    prepare();
  }

  void prepare() {
    var player = widget.element.player;
    var entry = widget.element.tikEntry;
    player?.addListener(() => playerListener(widget.position, player, entry));
    player?.onCurrentPosUpdate.listen(
        (event) => playProgressListen(player.value.duration, event, progressModel, playerIconModel, loadingModel));
    player?.onBufferPosUpdate
        .listen((event) => bufferPosUpdate(player.value.duration, event, progressModel, playerIconModel, loadingModel));
    player?.onBufferStateUpdate.listen((event) => bufferStateUpdate(event, loadingModel));
    player?.onBufferPercentUpdate.listen((percent) => bufferPercentUpdate(percent, player, entry, progressModel));
  }

  void bufferPercentUpdate(int percent, FijkPlayer player, VideoTikEntry entry, VideoPlayProgressModel progressModel) {
    if (!progressModel.isFirstStarted && entry.autoPlay && player.state != FijkState.started) {
      TikUtils.instance.start(player);
      progressModel.isFirstStarted = true;
    }
  }

  void bufferStateUpdate(bool state, VideoLoadingModel loadingModel) {
    loadingModel.isLoading = state;
    loadingModel.notifierLazy();
  }

  void bufferPosUpdate(Duration total, Duration duration, VideoPlayProgressModel progressModel,
      VideoPlayerIconModel playerIconModel, VideoLoadingModel loadingModel) {}

  void playProgressListen(Duration total, Duration duration, VideoPlayProgressModel progressModel,
      VideoPlayerIconModel playerIconModel, VideoLoadingModel loadingModel) {
    //播放进度
    var currentTime = duration.inMilliseconds.toDouble();
    var totalTime = total.inMilliseconds.toDouble();
    var progress = currentTime / totalTime;
    progressModel.notifyProgress(progress);
    progressModel.notifierLazy();
    if (progress >= 0.95 &&
        TikUtils.instance.getFrontState(widget.tabIndex) &&
        TikUtils.instance.index == widget.position) {
      var player = widget.element.player;
      player?.pause().then((value) {
        player.seekTo(widget.initSeek).then((value) {
          player.start();
        });
      });
    }
    //播放状态
    if (playerIconModel.isPlaying == false) {
      playerIconModel.isPlaying = true;
      playerIconModel.notifierLazy();
    }
    //加载状态
    if (loadingModel.isLoading == true) {
      loadingModel.isLoading = false;
      loadingModel.notifierLazy();
    }
  }

  void playerListener(int pos, FijkPlayer player, VideoTikEntry entry) {
    if (TikUtils.instance.getFrontState(widget.tabIndex)) {
      if (TikUtils.instance.index == pos) {
        if (player.state == FijkState.prepared) {
          player.seekTo(widget.initSeek).then((value) {
            player.start();
          });
        }
      } else {
        TikUtils.instance.pause(player);
        playerIconModel.isPlaying = false;
        playerIconModel.notifierLazy();
      }
    } else {
      TikUtils.instance.pause(player);
      playerIconModel.isPlaying = false;
      playerIconModel.notifierLazy();
    }
  }

  @override
  void dispose() {
    super.dispose();
    CloudEBus.instance.dispatch("playIconSetting");
  }

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      child: Stack(
        children: <Widget>[
          Stack(
            children: <Widget>[
              Container(
                height: double.infinity,
                width: double.infinity,
                alignment: Alignment.center,
                color: const Color(0xff020210),
                child: buildVideo(),
              ),
            ],
          ),
          buildFavoriteAnimationView(),
          // buildRightView(),
          buildPlayWidget(),
          buildLoadingWidget(),
          Positioned(
            child: buildPlayProgress(),
            left: 0,
            bottom: 0,
            right: 0,
          ),
        ],
      ),
      onTapDown: (detail) {
        setState(() {
          if (canAddFavorite) {
            icons.add(detail.globalPosition);
            widget.listener?.onFavoriteCall();
            justAddFavorite = true;
          } else {
            justAddFavorite = false;
          }
        });
      },
      onTapUp: (detail) {
        timer?.cancel();
        var delay = canAddFavorite ? 500 : 600;
        timer = Timer(Duration(milliseconds: delay), () {
          canAddFavorite = false;
          timer = null;
          if (!justAddFavorite) {
            singleTapCall();
          }
        });
        canAddFavorite = true;
      },
    );
  }

  Widget buildVideo() {
    if (widget.element.player == null) {
      return const SizedBox();
    }
    return Center(
      child: FijkView(
        width: double.infinity,
        height: double.infinity,
        fit: widget.fit,
        player: widget.element.player!,
        color: Colors.transparent,
        panelBuilder: (_, __, ___, ____, _____) => const SizedBox(),
      ),
    );
  }

  void singleTapCall() {
    var player = widget.element.player;
    if (player?.state == FijkState.started) {
      TikUtils.instance.pause(player);
      playerIconModel.isPlaying = false;
    } else {
      TikUtils.instance.start(player);
      playerIconModel.isPlaying = true;
    }
    playerIconModel.notifierLazy();
  }

  Widget buildPlayProgress() {
    return StateUtils.instance.subscribeNotifierWidget<VideoPlayProgressModel>(progressModel, (context, model) {
      return LinearPercentIndicator(
        width: MediaUtils.instance.screenWidth,
        lineHeight: 2,
        percent: model.currentProgress,
        backgroundColor: widget.progressBackgroundColor,
        progressColor: widget.progressColor,
        padding: EdgeInsets.zero,
      );
    });
  }

  // Widget buildRightView() {
  //   return Container(
  //     alignment: Alignment.centerRight,
  //     child: Padding(
  //       padding: const EdgeInsets.only(right: 20, bottom: 60),
  //       child: Column(
  //         mainAxisSize: MainAxisSize.min,
  //         children: [
  //           // buildPortrait(),
  //           // buildCollect(),
  //           // buildShare(),
  //         ],
  //       ),
  //     ),
  //   );
  // }

  Widget buildShare() {
    return StateUtils.instance.subscribeNotifierWidget<ShareModel>(shareModel, (context, model) {
      return Padding(
        padding: const EdgeInsets.only(top: 12),
        child: GestureDetector(
          child: Image.asset(
            "assets/images/share_icon.png",
            package: "eyflutter_tik_video",
            width: 42,
            height: 42,
          ),
          onTap: () => widget.listener?.onShareCall(model),
        ),
      );
    });
  }

  Widget buildCollect() {
    return StateUtils.instance.subscribeNotifierWidget<CollectModel>(collectModel, (context, model) {
      //这里根据isCollect替换收藏图片
      return Padding(
        padding: const EdgeInsets.only(top: 12),
        child: GestureDetector(
          child: Image.asset(
            "assets/images/love_icon.png",
            package: "eyflutter_tik_video",
            width: 42,
            height: 42,
          ),
          onTap: () => widget.listener?.onCollectCall(model),
        ),
      );
    });
  }

  Widget buildPortrait() {
    return StateUtils.instance.subscribeNotifierWidget<PortraitModel>(portraitModel, (context, model) {
      return Container(
        width: 52,
        height: 52,
        decoration: BoxDecoration(
          border: Border.all(color: Colors.white, width: 0.8),
          borderRadius: const BorderRadius.all(Radius.circular(28)),
          color: Colors.white,
        ),
        child: GestureDetector(
          child: Container(
            width: 42,
            height: 42,
            alignment: Alignment.center,
            child: Image.asset(
              "assets/images/ey_head_icon.png",
              package: "eyflutter_tik_video",
              width: 42,
              height: 42,
            ),
          ),
          onTap: () => widget.listener?.onPortraitCall(model),
        ),
      );
    });
  }

  Widget buildFavoriteAnimationView() {
    return Stack(
      children: icons
          .map<Widget>(
            (p) => FavoriteAnimationIcon(
              key: Key(p.toString()),
              position: p,
              favoriteComplete: () {
                icons.remove(p);
              },
              size: 50,
            ),
          )
          .toList(),
    );
  }

  Widget buildLoadingWidget() {
    return StateUtils.instance.subscribeNotifierWidget<VideoLoadingModel>(loadingModel, (context, model) {
      return Visibility(
        child: const Align(
          alignment: Alignment.center,
          child: CircularProgressIndicator(
            strokeWidth: 8,
            backgroundColor: Colors.grey,
            valueColor: AlwaysStoppedAnimation<Color>(Colors.blueGrey),
          ),
        ),
        visible: model.isLoading,
      );
    });
  }

  Widget buildPlayWidget() {
    return StateUtils.instance.subscribeNotifierWidget<VideoPlayerIconModel>(playerIconModel, (context, model) {
      return Visibility(
        child: Align(
          alignment: Alignment.center,
          child: GestureDetector(
            child: Image.asset(
              "assets/images/ey_playing_icon.png",
              package: "eyflutter_tik_video",
              width: 58,
              height: 58,
            ),
            onTap: () => playIconTapCall(),
          ),
        ),
        visible: !model.isPlaying,
      );
    });
  }

  void playIconTapCall() {
    var player = widget.element.player;
    if (player?.state == FijkState.started) {
      player?.pause();
      playerIconModel.isPlaying = false;
    } else {
      if (player?.state == FijkState.prepared ||
          player?.state == FijkState.paused ||
          player?.state == FijkState.completed) {
        player?.start();
        playerIconModel.isPlaying = true;
      } else {
        playerIconModel.isPlaying = false;
      }
    }
    playerIconModel.notifierLazy();
  }

  void onPlayIconSetting(bool isHidePlayIcon) {
    if (isHidePlayIcon) {
      playerIconModel.isPlaying = true;
    } else {
      playerIconModel.isPlaying = false;
    }
    playerIconModel.notifierLazy();
  }
}
