import 'package:eyflutter_tik_video/models/video_tik_list_model.dart';
import 'package:eyflutter_tik_video/tiklist/tik_video_gesture.dart';
import 'package:eyflutter_tik_video/utils/tik_utils.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class VideoTikListView extends StatefulWidget {
  /// 构建回调
  final Function(VideoTikListModel model) buildCall;

  /// 视频填充方式
  final FijkFit fit;

  /// 进度条颜色
  final Color progressColor;

  /// 进度条背景
  final Color progressBackgroundColor;

  final TikVideoListener? listener;

  final int initSeek;

  final int tabIndex;

  const VideoTikListView({
    Key? key,
    required this.buildCall,
    required this.fit,
    required this.progressColor,
    required this.progressBackgroundColor,
    this.listener,
    required this.initSeek,
    required this.tabIndex,
  }) : super(key: key);

  @override
  _VideoTikListViewState createState() => _VideoTikListViewState();
}

class _VideoTikListViewState extends LifecycleState<VideoTikListView> {
  VideoTikListModel listModel = VideoTikListModel();

  @override
  String get viewName => "videoTikList";

  @override
  void initState() {
    super.initState();
    widget.buildCall.call(listModel);
  }

  @override
  void onPause(String routeName, Map<String, dynamic> arguments) {
    TikUtils.instance.setFrontState(TikUtils.instance.currentTabIndex, false);
    TikUtils.instance.pause(listModel.currentItem()?.player);
  }

  @override
  void onResume(String routeName, Map<String, dynamic> arguments) {
    TikUtils.instance.setFrontState(TikUtils.instance.currentTabIndex, true);
    TikUtils.instance.start(listModel.currentItem()?.player);
  }

  @override
  void onDestroy(String routeName, Map<String, dynamic> arguments) {
    listModel.release();
  }

  @override
  Widget build(BuildContext context) {
    super.build(context);
    return StateUtils.instance.subscribeNotifierWidget<VideoTikListModel>(listModel, (context, model) {
      return PageView.builder(
        pageSnapping: true,
        physics: const ClampingScrollPhysics(),
        scrollDirection: Axis.vertical,
        itemCount: model.count,
        itemBuilder: (context, index) {
          return buildPageItem(context, index, model);
        },
        onPageChanged: (int page) => model.videoTurn(page),
      );
    });
  }

  Widget buildPageItem(BuildContext context, int index, VideoTikListModel model) {
    var ele = model.element(index);
    return TikVideoGesture(
      key: Key("${ele.tikEntry.videoUrl}$index"),
      buildCall: () {},
      progressColor: widget.progressColor,
      progressBackgroundColor: widget.progressBackgroundColor,
      fit: widget.fit,
      element: ele,
      listener: widget.listener,
      position: index,
      initSeek: widget.initSeek,
      tabIndex: widget.tabIndex,
    );
  }
}
