import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:fijkplayer/fijkplayer.dart';
import 'package:flutter/material.dart';

class TikUtils {
  factory TikUtils() => _getInstance();

  static TikUtils get instance => _getInstance();
  static TikUtils? _instance;

  static TikUtils _getInstance() {
    return _instance ??= TikUtils._internal();
  }

  TikUtils._internal();

  int currentTabIndex = 0;

  Map<int, ValueNotifier<int>> indexMap = {};

  Map<int, bool> frontMap = {};

  void setIndex(int index) {
    var notifier = indexMap[currentTabIndex];
    if (notifier == null) {
      indexMap[currentTabIndex] = ValueNotifier<int>(index);
    } else {
      notifier.value = index;
    }
  }

  int get index => indexMap[currentTabIndex]?.value ?? 0;

  int getIndex(int tabIndex) {
    return indexMap[tabIndex]?.value ?? 0;
  }

  void setFrontState(int tabIndex, bool state) {
    frontMap[tabIndex] = state;
  }

  bool get isFrontPage => frontMap[currentTabIndex] ?? false;

  bool getFrontState(int tabIndex) {
    return frontMap[tabIndex] ?? false;
  }

  /// 播放视频
  void start(FijkPlayer? player) {
    if (player?.state == FijkState.prepared ||
        player?.state == FijkState.paused ||
        player?.state == FijkState.completed) {
      player?.start();
    }
  }

  /// 复原并暂停
  void recoveryPause(FijkPlayer? player) {
    if (player?.state == FijkState.started) {
      var duration = player?.value.duration ?? Duration.zero;
      if (duration.inMilliseconds > 1) {
        player?.seekTo(1);
      }
      player?.pause();
    }
  }

  /// 暂停
  void pause(FijkPlayer? player) {
    if (player?.state == FijkState.started) {
      player?.pause();
    }
  }

  /// 视频切换
  void toggle(FijkPlayer? player) {
    if (player?.state == FijkState.started) {
      player?.pause();
      CloudEBus.instance.dispatch("playIconSetting", params: true);
    } else if (player?.state == FijkState.prepared ||
        player?.state == FijkState.paused ||
        player?.state == FijkState.completed) {
      player?.start();
      CloudEBus.instance.dispatch("playIconSetting", params: false);
    }
  }
}
